# -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-
# Copyright 2014 Canonical
# Author: Omer Akram <omer.akram@canonical.com>
#
# This file is part of dialer-app.
#
# dialer-app is free software: you can redistribute it and/or modify it
# under the terms of the GNU General Public License version 3, as published
# by the Free Software Foundation.

"""Tests for the Dialer App"""

from autopilot.platform import model
from autopilot.matchers import Eventually
from testtools import skipIf
from testtools.matchers import Equals
from url_dispatcher_testability import (
    fake_dispatcher,
    fixture_setup as url_dispatcher_fixtures
)

from address_book_app.address_book import data
from dialer_app.tests import DialerAppTestCase
from dialer_app import fixture_setup
from dialer_app import ListItemWithActions


class TestCallLogs(DialerAppTestCase):
    """Tests for the call log panel."""

    def setUp(self):
        # set the fixtures before launching the app
        testability_environment = fixture_setup.TestabilityEnvironment()
        self.useFixture(testability_environment)
        fill_history = fixture_setup.FillCustomHistory()
        self.useFixture(fill_history)
        self.fake_url_dispatcher = url_dispatcher_fixtures.FakeURLDispatcher()
        self.useFixture(self.fake_url_dispatcher)
        memory_backend = fixture_setup.UseMemoryContactBackend()
        self.useFixture(memory_backend)
        preload_data = fixture_setup.PreloadVcards()
        self.useFixture(preload_data)

        # now launch the app
        super().setUp()
        self.main_view.dialer_page.reveal_bottom_edge_page()

    def _get_main_view(self, proxy_object):
        return proxy_object.wait_select_single('QQuickView')

    def get_last_dispatch_url_call_parameter(self):
        try:
            fake = self.fake_url_dispatcher
            return fake.get_last_dispatch_url_call_parameter()
        except fake_dispatcher.FakeDispatcherException:
            return None

    @skipIf(model() == 'Desktop', 'only run on Ubuntu touch platforms')
    def test_call_log_item_opens_messaging(self):
        """Ensure tapping on 'send text message' item of a call log opens
        the messaging app.

        """
        delegate = self.main_view.wait_select_single(
            ListItemWithActions.HistoryDelegate, objectName='historyDelegate0')
        delegate.send_message()

        self.assertThat(
            self.get_last_dispatch_url_call_parameter,
            Eventually(Equals('message:///800')))

    def test_add_new_contact_from_log(self):
        """Ensure tapping on 'add new contact' item of a call log opens
        the address-book app to allow adding new contact.

        """
        delegate = self.main_view.wait_select_single(
            ListItemWithActions.HistoryDelegate, objectName='historyDelegate0')
        delegate.add_contact()

        contacts_page = self.main_view.contacts_page
        self.assertThat(
            contacts_page.phoneToAdd, Eventually(Equals('800')))

        # click add new button
        contacts_page.click_add_new()

        # wait page be ready for edit
        contact_editor_page = self.main_view.contact_editor_page
        self.main_view.contact_editor_page.wait_get_focus('phones')

        # fill contact name
        test_contact = data.Contact('FirstName', 'LastName')
        test_contact.professional_details = []
        contact_editor_page.fill_form(test_contact)

        # save contact
        contact_editor_page.save()

        # contact view will appear with the new contact data
        contact_view_page = contacts_page.open_contact(3)
        self.assertThat(contact_view_page.visible, Eventually(Equals(True)))

        # check if contact contains the new phone number
        phone_group = contact_view_page.select_single(
            'ContactDetailGroupWithTypeView',
            objectName='phones')
        self.assertThat(phone_group.detailsCount, Eventually(Equals(1)))

        # check if the new value is correct
        phone_label_1 = contact_view_page.select_single(
            "UCLabel",
            objectName="label_phoneNumber_0.0")
        self.assertThat(phone_label_1.text, Eventually(Equals('800')))

    def test_add_number_into_old_contact_from_log(self):
        """Ensure tapping on 'add new contact' item of a call log opens
        the address-book app to allow add the numbe into a contact

        """
        delegate = self.main_view.wait_select_single(
            ListItemWithActions.HistoryDelegate, objectName='historyDelegate0')
        delegate.add_contact()

        contacts_page = self.main_view.contacts_page
        self.assertThat(
            contacts_page.phoneToAdd, Eventually(Equals('800')))

        # click on first contact to add number
        contacts_page.click_contact(0)

        # wait page be ready for edit
        contact_editor_page = self.main_view.contact_editor_page
        contact_editor_page.wait_get_focus('phones')

        # save contact
        contact_editor_page.save()

        # contact view will appear with the new contact data
        contact_view_page = self.main_view.contact_view_page

        # check if contact contains the new phone number
        phone_group = contact_view_page.select_single(
            'ContactDetailGroupWithTypeView',
            objectName='phones')
        self.assertThat(phone_group.detailsCount, Eventually(Equals(2)))

        # check if the new value is correct
        phone_label_1 = contact_view_page.select_single(
            "UCLabel",
            objectName="label_phoneNumber_1.0")
        self.assertThat(phone_label_1.text, Eventually(Equals('800')))


class TestSwipeItemTutorial(DialerAppTestCase):
    """Tests for swipe item tutorial."""

    def setUp(self):
        # set the fixtures before launching the app
        testability_environment = fixture_setup.TestabilityEnvironment()
        self.useFixture(testability_environment)
        fill_history = fixture_setup.FillCustomHistory()
        self.useFixture(fill_history)

        # now launch the app
        super().setUp(firstLaunch=True)
        self.main_view.dialer_page.reveal_bottom_edge_page()

    def _get_main_view(self, proxy_object):
        return proxy_object.wait_select_single('QQuickView')

    def test_swipe_item_tutorial_appears(self):
        """Ensure that the swipe item tutorial appears on first launch"""
        swipe_item_demo = self.main_view.wait_select_single(
            'SwipeItemDemo', objectName='swipeItemDemo')

        self.assertThat(swipe_item_demo.enabled, Eventually(Equals(True)))
        self.assertThat(swipe_item_demo.necessary, Eventually(Equals(True)))
        got_it_button = swipe_item_demo.select_single(
            'Button',
            objectName='gotItButton')
        self.main_view._click_button(got_it_button)
        self.assertThat(swipe_item_demo.enabled, Eventually(Equals(False)))
        self.assertThat(swipe_item_demo.necessary, Eventually(Equals(False)))



# -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-
# Copyright 2012, 2013, 2014 Canonical
#
# This file is part of dialer-app.
#
# dialer-app is free software: you can redistribute it and/or modify it
# under the terms of the GNU General Public License version 3, as published
# by the Free Software Foundation.

"""Dialer App autopilot tests."""

import logging
import os
import tempfile

import fixtures
import ubuntuuitoolkit
from autopilot.matchers import Eventually
from autopilot.testcase import AutopilotTestCase
from testtools.matchers import Equals
from ubuntuuitoolkit import fixture_setup

import dialer_app
from dialer_app import helpers


logger = logging.getLogger(__name__)


# ensure we have an ofono account; we assume that we have these tools,
# otherwise we consider this a test failure (missing dependencies)
helpers.ensure_ofono_account()


class DialerAppTestCase(AutopilotTestCase):
    """A common test case class that provides several useful methods for
    Dialer App tests.

    """

    LOCAL_BINARY_PATH = 'src/dialer-app'
    # The path to the locally built binary, relative to the build directory.

    def setUp(self, firstLaunch=False):
        super().setUp()
        self.set_up_locale()
        self.create_config_file(firstLaunch)
        self.launch_application()

        self.assertThat(self.main_view.visible, Eventually(Equals(True)))

    def set_up_locale(self):
        # We set up the language to english to check the formatting of the
        # dialed number.
        self.useFixture(
            fixtures.EnvironmentVariable('LANGUAGE', newvalue='en')
        )
        self.useFixture(
            fixture_setup.InitctlEnvironmentVariable(LANGUAGE='en')
        )

    def launch_application(self):
        build_dir = os.environ.get('BUILD_DIR', None)
        if build_dir is not None:
            self.launch_built_application(build_dir)
        else:
            self.launch_installed_application()

    def launch_built_application(self, build_dir):
        binary_path = os.path.join(build_dir, self.LOCAL_BINARY_PATH)
        self.app = self.launch_test_application(
            binary_path,
            app_type='qt',
            emulator_base=ubuntuuitoolkit.UbuntuUIToolkitCustomProxyObjectBase
        )

    def launch_installed_application(self):
        self.app = self.launch_upstart_application(
            'dialer-app',
            emulator_base=ubuntuuitoolkit.UbuntuUIToolkitCustomProxyObjectBase
        )

    def create_config_file(self, firstLaunch=False):
        self.user_config_dir = tempfile.mkdtemp(suffix='', prefix='dialer-app')
        self.app_config_dir = (self.user_config_dir +
                               '/com.ubuntu.dialer-app/')
        os.makedirs(self.app_config_dir)
        config_file_path = (self.app_config_dir + '/DialerApp.conf')

        if firstLaunch:
            first_launch_flag = 'true'
        else:
            first_launch_flag = 'false'
        with open(config_file_path, 'w') as config_file:
            config_file.write(
                '[General]\nhintNecessary=%s\n' % (first_launch_flag))

        self.useFixture(
            fixtures.EnvironmentVariable('XDG_CONFIG_HOME',
                                         newvalue=self.user_config_dir)
        )
        self.useFixture(
            fixture_setup.InitctlEnvironmentVariable(
                XDG_CONFIG_HOME=self.user_config_dir)
        )

    @property
    def main_view(self):
        return self.app.wait_select_single(dialer_app.MainView)



# -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-
# Copyright 2015 Canonical
# Author: Renato Araujo Oliveira Filho <renato.filho@canonical.com>
#
# This file is part of dialer-app.
#
# dialer-app is free software: you can redistribute it and/or modify it
# under the terms of the GNU General Public License version 3, as published
# by the Free Software Foundation.

"""Tests for the Dialer App"""

from autopilot.matchers import Eventually
from testtools.matchers import Equals

from dialer_app.tests import DialerAppTestCase
from dialer_app import fixture_setup


class TestContacts(DialerAppTestCase):
    """Tests for the contacts interaction with the app."""

    def setUp(self):
        # set the fixtures before launching the app
        testability_environment = fixture_setup.TestabilityEnvironment()
        self.useFixture(testability_environment)
        memory_backend = fixture_setup.UseMemoryContactBackend()
        self.useFixture(memory_backend)
        preload_data = fixture_setup.PreloadVcards()
        self.useFixture(preload_data)

        # now launch the app
        super().setUp()

    def _get_main_view(self, proxy_object):
        return proxy_object.wait_select_single('QQuickView')

    def test_call_a_contact_from_contact_view(self):
        dialer_page = self.main_view.dialer_page
        dialer_page.click_contacts_button()
        contact_view_page = self.main_view.contacts_page.open_contact(0)

        contact_view_page.call_phone(0)
        entry = dialer_page._get_keypad_entry()
        self.assertThat(entry.value, Eventually(Equals('444-44')))



# -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-
# Copyright 2013, 2014 Canonical
#
# This file is part of dialer-app.
#
# dialer-app is free software: you can redistribute it and/or modify it
# under the terms of the GNU General Public License version 3, as published
# by the Free Software Foundation.

"""Dialer app autopilot custom proxy objects."""

import logging

import ubuntuuitoolkit
import time

from address_book_app.address_book import _common
from address_book_app import address_book
from autopilot import exceptions as autopilot_exceptions


class MainView(ubuntuuitoolkit.MainView):
    def __init__(self, *args):
        super().__init__(*args)
        self.logger = logging.getLogger(__name__)

    @property
    def dialer_page(self):
        return self.wait_select_single(DialerPage, greeterMode=False)

    @property
    def live_call_page(self):
        # wait until we actually have the calls before returning the live call
        self.hasCalls.wait_for(True)
        return self.wait_select_single(LiveCall, active=True)

    @property
    def contacts_page(self):
        return self._get_page(ContactsPage, 'contactsPage')

    @property
    def contact_editor_page(self):
        return self._get_page(DialerContactEditorPage, 'contactEditorPage')

    @property
    def contact_view_page(self):
        return self._get_page(DialerContactViewPage, 'contactViewPage')

    def get_first_log(self):
        return self.wait_select_single(objectName="historyDelegate0")

    def _click_button(self, button):
        """Generic way to click a button"""
        self.visible.wait_for(True)
        button.visible.wait_for(True)
        self.pointing_device.click_object(button)
        return button

    def _long_press(self, obj):
        """long press on object because press_duration is not honored on touch
        see bug #1268782

        :parameter obj: the object to long press on
        """

        self.pointing_device.move_to_object(obj)
        self.pointing_device.press()
        time.sleep(3)
        self.pointing_device.release()

    def check_ussd_error_dialog_visible(self):
        """Check if ussd error dialog is visible"""
        dialog = None
        try:
            dialog = self.wait_select_single(objectName="ussdErrorDialog")
        except:
            # it is ok to fail in this case
            return False

        return dialog.visible

    def check_ussd_progress_dialog_visible(self):
        """Check if ussd progress dialog is visible"""
        dialog = None
        try:
            dialog = self.wait_select_single(
                objectName="ussdProgressDialog")
        except:
            # it is ok to fail in this case
            return False

        return dialog.visible

    def _get_page(self, page_type, page_name):
        page = self.wait_select_single(
            page_type, objectName=page_name, active=True)
        return page


class LiveCall(MainView):

    def get_elapsed_call_time(self):
        """Return the elapsed call time"""
        return self.wait_select_single(objectName='stopWatch').elapsed

    def _get_hangup_button(self):
        """Return the hangup button"""
        return self.wait_select_single(objectName='hangupButton')

    def _get_call_hold_button(self):
        """Return the call holding button"""
        return self.wait_select_single(objectName='callHoldButton')

    def _get_swap_calls_button(self):
        """Return the swap calls button"""
        return self._get_call_hold_button()

    def _get_new_call_button(self):
        return self.wait_select_single(objectName='newCallButton')

    def get_multi_call_display(self):
        """Return the multi call display panel"""
        return self.wait_select_single(objectName='multiCallDisplay')

    def get_multi_call_item_for_number(self, number):
        """Return the multi call display item for the given number"""
        return self.wait_select_single(objectName='callDelegate',
                                       phoneNumber=number)

    def click_hangup_button(self):
        """Click and return the hangup page"""
        return self._click_button(self._get_hangup_button())

    def click_call_hold_button(self):
        """Click the call holding button"""
        return self._click_button(self._get_call_hold_button())

    def click_swap_calls_button(self):
        """Click the swap calls button"""
        return self._click_button(self._get_swap_calls_button())

    def click_new_call_button(self):
        """Click the new call button"""
        return self._click_button(self._get_new_call_button())


class DialerPage(MainView):

    def reveal_bottom_edge_page(self):
        """Bring the bottom edge page to the screen"""
        try:
            start_x = (
                self.globalRect.x +
                (self.globalRect.width * 0.5))
            # Start swiping from the top of the component because after some
            # seconds it gets almost fully hidden. The center will be out of
            # view.
            start_y = self.globalRect.y + self.height - 3
            stop_y = start_y - (self.height * 0.7)
            self.pointing_device.drag(start_x, start_y, start_x, stop_y,
                                      rate=2)
            self.bottomEdgeCommitted.wait_for(True)
        except autopilot_exceptions.StateNotFoundError:
            self.logger.error('BottomEdge element not found.')
            raise

    def _get_keypad_entry(self):
        return self.wait_select_single("KeypadEntry")

    def _get_keypad_keys(self):
        return self.select_many("KeypadButton")

    def _get_keypad_key(self, number):
        buttons_dict = {
            "0": "buttonZero",
            "1": "buttonOne",
            "2": "buttonTwo",
            "3": "buttonThree",
            "4": "buttonFour",
            "5": "buttonFive",
            "6": "buttonSix",
            "7": "buttonSeven",
            "8": "buttonEight",
            "9": "buttonNine",
            "*": "buttonAsterisk",
            "#": "buttonHash",
        }
        return self.wait_select_single("KeypadButton",
                                       objectName=buttons_dict[number])

    def _get_erase_button(self):
        """Return the erase button"""
        return self.wait_select_single("CustomButton",
                                       objectName="eraseButton")

    def _get_call_button(self):
        """Return the call button"""
        return self.wait_select_single(objectName="callButton")

    def click_call_button(self):
        """Click and return the call button"""
        return self._click_button(self._get_call_button())

    def click_erase_button(self):
        """Click the erase button"""
        self._click_button(self._get_erase_button())

    def click_keypad_button(self, keypad_button):
        """click the keypad button

        :param keypad_button: the clicked keypad_button
        """
        self._click_button(keypad_button)

    def trigger_copy_and_paste(self):
        """Invoke the copy and paste popup"""
        self._long_press(self._get_keypad_entry())

    def trigger_select_all(self):
        """Trigger select all"""
        button = self.get_root_instance().wait_select_single(
                        'UCAbstractButton',
                        objectName="select_all_button")
        self._click_button(button)

    def trigger_copy(self):
        """Trigger copy"""
        button = self.get_root_instance().wait_select_single(
                         'UCAbstractButton',
                         objectName="copy_button")
        self._click_button(button)

    def trigger_paste(self):
        """Trigger paste"""
        button = self.get_root_instance().wait_select_single(
                          'UCAbstractButton',
                          objectName="paste_button")
        self._click_button(button)

    def trigger_cut(self):
        """Trigger cut"""
        button = self.get_root_instance().wait_select_single(
                          'UCAbstractButton',
                          objectName="cut_button")
        self._click_button(button)

    def dial_number(self, number, formattedNumber):
        """Dial given number (string) on the keypad and return keypad entry

        :param number: the number to dial
        """
        for digit in number:
            button = self._get_keypad_key(digit)
            self.click_keypad_button(button)

        entry = self._get_keypad_entry()
        entry.value.wait_for(formattedNumber)
        return entry

    def call_number(self, number, formattedNumber):
        """Dial number and call return call_button"""
        self.dial_number(number, formattedNumber)
        self.click_call_button()
        return self.get_root_instance().wait_select_single(LiveCall)

    def click_header_action(self, action):
        """Click the action 'action' on the header"""
        action = self.wait_select_single(objectName='%s_button' % action)
        self.pointing_device.click_object(action)

    def click_contacts_button(self):
        self.click_header_action('contacts')


class DialerContactViewPage(address_book.ContactViewPage):
    """Autopilot custom proxy object for DialerContactViewPage components."""

    def call_phone(self, index):
        phone_group = self.select_single(
            'ContactDetailGroupWithTypeView',
            objectName='phones')

        call_buttons = phone_group.select_many(
            "ActionButton",
            objectName="tel-contact")
        self.pointing_device.click_object(call_buttons[index])


class DialerContactEditorPage(address_book.ContactEditorPage):
    """Autopilot custom proxy object for DialerContactEditorPage components."""

    def click_action_button(self, action_name):
        actions = self.select_many(objectName='%s_button' % action_name)
        for action in actions:
            if action.enabled:
                self.pointing_device.click_object(action)
                return

        raise autopilot_exceptions.StateNotFoundError(action_name)

    def save(self):
        """
        Press the 'Save' button
        """
        self.click_action_button('save')


class ContactsPage(_common.PageWithHeader):
    """Autopilot custom proxy object for ContactsPage components."""

    def _click_button(self, button):
        """Generic way to click a button"""
        self.visible.wait_for(True)
        button.visible.wait_for(True)
        self.pointing_device.click_object(button)
        return button

    def _get_add_new_button(self):
        """Return the add-new button"""
        return self.wait_select_single('ContactListButtonDelegate',
                                       objectName='addNewButton')

    def click_add_new(self):
        self._click_button(self._get_add_new_button())

    def click_contact(self, index):
        contact_delegate = self._get_contact_delegate(index)
        self.pointing_device.click_object(contact_delegate)

    def open_contact(self, index):
        self.click_contact(index)
        return self.get_root_instance().select_single(
            DialerContactViewPage, objectName='contactViewPage')

    def _get_contact_delegate(self, index):
        contact_delegates = self._get_sorted_contact_delegates()
        return contact_delegates[index]

    def _get_sorted_contact_delegates(self):
        contact_delegates = self.select_many('ContactDelegate', visible=True)
        return sorted(
            contact_delegates, key=lambda delegate: delegate.globalRect.y)



# -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-
#
# Copyright 2014-2015 Canonical Ltd.
# Authors: Omer Akram <omer.akram@canonical.com>
#          Gustavo Pichorim Boiko <gustavo.boiko@canonical.com>
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License version 3, as published
# by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.

import subprocess
import sys
import time
import dbus
import tempfile
import os
import shutil


def wait_for_incoming_call():
    """Wait up to 5 s for an incoming phone call"""

    timeout = 10
    while timeout >= 0:
        out = subprocess.check_output(
            ['/usr/share/ofono/scripts/list-calls'],
            stderr=subprocess.PIPE,
            universal_newlines=True)
        if 'State = incoming' in out or 'State = waiting' in out:
            break
        timeout -= 1
        time.sleep(0.5)
    else:
        raise RuntimeError('timed out waiting for incoming phonesim call')


def run_script(script):
    # prepare and send a Qt GUI script to phonesim, over its private D-BUS
    # set up by ofono-phonesim-autostart
    script_dir = tempfile.mkdtemp(prefix="phonesim_script")
    os.chmod(script_dir, 0o755)
    with open(os.path.join(script_dir, "script.js"), "w") as f:
        f.write(script)

    with open("/run/lock/ofono-phonesim-dbus.address") as f:
        phonesim_bus = f.read().strip()
    bus = dbus.bus.BusConnection(phonesim_bus)
    script_proxy = bus.get_object("org.ofono.phonesim", "/")
    script_proxy.SetPath(script_dir)
    script_proxy.Run("script.js")
    shutil.rmtree(script_dir)


def invoke_incoming_call(caller):
    """Receive an incoming call from the given caller

    :parameter caller: the phone number calling
    """

    run_script("""tabCall.gbIncomingCall.leCaller.text = "%s";
tabCall.gbIncomingCall.pbIncomingCall.click();
""" % (caller))


def hangup_call():
    """Hangup an existing call. This only works when there is only one call"""

    run_script("""tabCall.twCallMgt.selectColumn(0);
tabCall.pbHangup.click();""")


def accept_incoming_call():
    """Accept an existing incoming call"""
    subprocess.check_call(
        [
            "dbus-send", "--session", "--print-reply",
            "--dest=com.canonical.Approver", "/com/canonical/Approver",
            "com.canonical.TelephonyServiceApprover.AcceptCall"
        ], stdout=subprocess.PIPE)


def get_phonesim():
    bus = dbus.SystemBus()
    try:
        manager = dbus.Interface(bus.get_object('org.ofono', '/'),
                                 'org.ofono.Manager')
    except dbus.exceptions.DBusException:
        return False

    modems = manager.GetModems()

    for path, properties in modems:
        if path == '/phonesim':
            return properties

    return None


def is_phonesim_running():
    """Determine whether we are running with phonesim."""
    phonesim = get_phonesim()
    return phonesim is not None


def ensure_ofono_account():
    # oFono modems are now set online by NetworkManager, so for the tests
    # we need to manually put them online.
    subprocess.check_call(['/usr/share/ofono/scripts/enable-modem',
                           '/phonesim'])
    subprocess.check_call(['/usr/share/ofono/scripts/online-modem',
                           '/phonesim'])

    # wait until the modem is actually online
    for index in range(10):
        phonesim = get_phonesim()
        if phonesim['Online'] == 1:
            break
        time.sleep(1)
    else:
        raise RuntimeError("oFono phone simulator didn't get online.")

    # this is a bit drastic, but sometimes mission-control-5 won't recognize
    # clients installed after it was started, so, we make sure it gets
    # restarted
    subprocess.check_call(['pkill', '-9', 'mission-control'])

    if not _is_ofono_account_set():
        subprocess.check_call(['ofono-setup'])
        if not _is_ofono_account_set():
            sys.stderr.write('ofono-setup failed to create ofono account!\n')
            sys.exit(1)


def _is_ofono_account_set():
    mc_tool = subprocess.Popen(
        [
            'mc-tool',
            'list',
        ], stdout=subprocess.PIPE, universal_newlines=True)
    mc_accounts = mc_tool.communicate()[0]
    return 'ofono/ofono/account' in mc_accounts



# -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-
# Copyright 2015 Canonical
# Author: Gustavo Pichorim Boiko <gustavo.boiko@canonical.com>
#
# This file is part of dialer-app.
#
# dialer-app is free software: you can redistribute it and/or modify it
# under the terms of the GNU General Public License version 3, as published
# by the Free Software Foundation.

"""Multiple calls tests for the Dialer App using ofono-phonesim"""

import time

from autopilot.matchers import Eventually
from testtools.matchers import Equals
from testtools import skipUnless

from dialer_app.tests import DialerAppTestCase
from dialer_app import helpers
from dialer_app import fixture_setup


@skipUnless(helpers.is_phonesim_running(),
            "this test needs to run under with-ofono-phonesim")
class TestMultiCalls(DialerAppTestCase):
    """Tests for simulated phone calls."""

    def setUp(self):
        empty_history = fixture_setup.UseEmptyHistory()
        self.useFixture(empty_history)
        phonesim_modem = fixture_setup.UsePhonesimModem()
        self.useFixture(phonesim_modem)
        notification_mock = fixture_setup.MockNotificationSystem()
        self.useFixture(notification_mock)
        super().setUp()

    def tearDown(self):
        super().tearDown()

    @property
    def history_list(self):
        # because of the object tree, more than just one item is returned, but
        # all references point to the same item, so take the first
        return self.app.select_many(objectName="historyList")[0]

    def get_history_for_number(self, number):
        # because of the bottom edge tree structure, multiple copies of the
        # same item are returned, so just use the first one
        return self.history_list.select_many(
            "HistoryDelegate", phoneNumber=number)[0]

    def place_calls(self, numbers):
        for number in numbers:
            helpers.invoke_incoming_call(number)
            helpers.wait_for_incoming_call()
            time.sleep(1)
            helpers.accept_incoming_call()
            time.sleep(1)

    def test_multi_call_panel(self):
        """Make sure the multi call panel is visible when two calls are
           available"""
        firstNumber = '11111111'
        secondNumber = '22222222'

        # place the calls
        self.place_calls([firstNumber, secondNumber])

        # now ensure that the multi-call panel is visible
        multi_call = self.main_view.live_call_page.get_multi_call_display()
        self.assertThat(multi_call.visible, Eventually(Equals(True)))

        # hangup one call
        self.main_view.live_call_page.click_hangup_button()
        self.assertThat(multi_call.visible, Eventually(Equals(False)))

        # and the other one
        self.main_view.live_call_page.click_hangup_button()

    def test_swap_calls(self):
        """Check that pressing the swap calls button change the call status"""
        firstNumber = '11111111'
        secondNumber = '22222222'

        # place the calls
        self.place_calls([firstNumber, secondNumber])

        live_call = self.main_view.live_call_page
        firstCallItem = live_call.get_multi_call_item_for_number(firstNumber)
        secondCallItem = live_call.get_multi_call_item_for_number(secondNumber)

        # check the current status
        self.assertThat(firstCallItem.active, Eventually(Equals(False)))
        self.assertThat(secondCallItem.active, Eventually(Equals(True)))

        # now swap the calls
        live_call.click_swap_calls_button()

        # and make sure the calls changed
        self.assertThat(firstCallItem.active, Eventually(Equals(True)))
        self.assertThat(secondCallItem.active, Eventually(Equals(False)))

        # hangup the calls
        self.main_view.live_call_page.click_hangup_button()
        time.sleep(1)
        self.main_view.live_call_page.click_hangup_button()

    def test_swap_and_hangup(self):
        """Check that after swapping calls and hanging up the correct call
           stays active"""
        firstNumber = '11111111'
        secondNumber = '22222222'

        # place the calls
        self.place_calls([firstNumber, secondNumber])

        # at this point the calls should be like this:
        #  - 11111111: held
        #  - 22222222: active
        # swap the calls then
        self.main_view.live_call_page.click_swap_calls_button()

        # - 11111111: active
        # - 22222222: held
        self.assertThat(
            self.main_view.live_call_page.caller,
            Eventually(Equals(firstNumber)))

        # hangup the active call
        self.main_view.live_call_page.click_hangup_button()

        # and check that the remaining call is the one that was held
        self.assertThat(
            self.main_view.live_call_page.caller,
            Eventually(Equals(secondNumber)))

        # and hangup the remaining call too
        self.main_view.live_call_page.click_hangup_button()



# -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-
#
# Copyright 2014 Canonical Ltd.
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License version 3, as published
# by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.

"""Set up and clean up fixtures for the Inter-app integration tests."""


import fixtures
import subprocess
import os
import shutil
import dbusmock
from autopilot.platform import model
import dbus
from ubuntuuitoolkit import fixture_setup


class TestabilityEnvironment(fixtures.Fixture):

    def setUp(self):
        super().setUp()
        self._set_testability_environment_variable()
        self.addCleanup(self._reset_environment_variable)

    def _set_testability_environment_variable(self):
        """Make sure every app loads the testability driver."""
        subprocess.call(
            [
                '/sbin/initctl',
                'set-env',
                '--global',
                'QT_LOAD_TESTABILITY=1'
            ]
        )

    def _reset_environment_variable(self):
        """Resets the previously added env variable."""
        subprocess.call(
            [
                '/sbin/initctl',
                'unset-env',
                'QT_LOAD_TESTABILITY'
            ]
        )


class FillCustomHistory(fixtures.Fixture):

    history_db = "history.sqlite"
    data_sys = "/usr/lib/python3/dist-packages/dialer_app/data/"
    data_local = "dialer_app/data/"
    database_path = '/tmp/' + history_db

    prefilled_history_local = os.path.join(data_local, history_db)
    prefilled_history_system = os.path.join(data_sys, history_db)

    def setUp(self):
        super(FillCustomHistory, self).setUp()
        self.addCleanup(self._clear_test_data)
        self.addCleanup(self._kill_service_to_respawn)
        self._clear_test_data()
        self._prepare_history_data()
        self._kill_service_to_respawn()
        self._start_service_with_custom_data()

    def _prepare_history_data(self):
        if os.path.exists(self.prefilled_history_local):
            shutil.copy(self.prefilled_history_local, self.database_path)
        else:
            shutil.copy(self.prefilled_history_system, self.database_path)

    def _clear_test_data(self):
        if os.path.exists(self.database_path):
            os.remove(self.database_path)

    def _kill_service_to_respawn(self):
        subprocess.call(['pkill', 'history-daemon'])

    def _start_service_with_custom_data(self):
        os.environ['HISTORY_SQLITE_DBPATH'] = self.database_path
        with open(os.devnull, 'w') as devnull:
            subprocess.Popen(['history-daemon'], stderr=devnull)


class UseEmptyHistory(FillCustomHistory):
    database_path = ':memory:'

    def setUp(self):
        super(UseEmptyHistory, self).setUp()

    def _prepare_history_data(self):
        # just avoid doing anything
        self.database_path = ':memory:'

    def _clear_test_data(self):
        # don't do anything
        self.database_path = ''


class UsePhonesimModem(fixtures.Fixture):

    def setUp(self):
        super().setUp()

        # configure the cleanups
        self.addCleanup(self._hangupLeftoverCalls)
        self.addCleanup(self._restoreModems)

        self._switchToPhonesim()

    def _switchToPhonesim(self):
        # make sure the modem is running on phonesim
        subprocess.call(['mc-tool', 'update', 'ofono/ofono/account0',
                         'string:modem-objpath=/phonesim'])
        subprocess.call(['mc-tool', 'reconnect', 'ofono/ofono/account0'])

    def _hangupLeftoverCalls(self):
        # ensure that there are no leftover calls in case of failed tests
        subprocess.call(["/usr/share/ofono/scripts/hangup-all", "/phonesim"])

    def _restoreModems(self):
        # set the modem objpath in telepathy-ofono to the real modem
        subprocess.call(['mc-tool', 'update', 'ofono/ofono/account0',
                         'string:modem-objpath=/ril_0'])
        subprocess.call(['mc-tool', 'reconnect', 'ofono/ofono/account0'])


class UseMemoryContactBackend(fixtures.Fixture):

    def setUp(self):
        super().setUp()
        self.useFixture(
            fixtures.EnvironmentVariable(
                'QTCONTACTS_MANAGER_OVERRIDE', newvalue='memory')
        )
        self.useFixture(
            fixture_setup.InitctlEnvironmentVariable(
                QTCONTACTS_MANAGER_OVERRIDE='memory')
        )


class PreloadVcards(fixtures.Fixture):
    AUTOPILOT_DIR = "/usr/lib/python3/dist-packages/dialer_app/"
    VCARD_PATH_BIN = ("%s/testdata/vcard.vcf" % AUTOPILOT_DIR)
    VCARD_PATH_DEV = os.path.abspath("../data/vcard.vcf")

    def setUp(self):
        super().setUp()
        vcard_full_path = PreloadVcards.VCARD_PATH_BIN
        if os.path.isfile(PreloadVcards.VCARD_PATH_DEV):
            vcard_full_path = PreloadVcards.VCARD_PATH_DEV

        print("Loading contacts from: %s" % vcard_full_path)
        self.useFixture(
            fixtures.EnvironmentVariable(
                'QTCONTACTS_PRELOAD_VCARD', newvalue=vcard_full_path)
        )
        self.useFixture(
            fixture_setup.InitctlEnvironmentVariable(
                QTCONTACTS_PRELOAD_VCARD=vcard_full_path)
        )


class MockNotificationSystem(fixtures.Fixture):

    def setUp(self):
        super().setUp()

        # only mock the notification system on desktop, on ubuntu touch the
        # notification dbus service is embedded into unity
        if model() == 'Desktop':
            self.addCleanup(self._stop_mock)
            self._kill_notification_service()
            # start the mock service
            (self.process, self.obj) = \
                dbusmock.DBusTestCase.spawn_server_template(
                    'notification_daemon')
        else:
            self.addCleanup(self._clear_existing_notifications)

    def _stop_mock(self):
        self.process.terminate()
        self.process.wait()

    def _kill_notification_service(self):
        """Kill the notification daemon."""
        subprocess.call(['pkill', '-f', 'notify-osd'])

    def _clear_existing_notifications(self):
        """Kill processes that might be displaying notifications"""
        bus = dbus.SessionBus()
        indicator = bus.get_object('com.canonical.TelephonyServiceIndicator',
                                   '/com/canonical/TelephonyServiceIndicator')
        indicator.ClearNotifications()



# -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-
# Copyright 2013 Canonical
#
# This file is part of dialer-app.
#
# dialer-app is free software: you can redistribute it and/or modify it
# under the terms of the GNU General Public License version 3, as published
# by the Free Software Foundation.

"""Dialer app autopilot custom proxy objects."""

from ubuntuuitoolkit._custom_proxy_objects import _common


class ListItemWithActions(_common.UbuntuUIToolkitCustomProxyObjectBase):

    def _drag_pointing_device_to_delete(self):
        x, y, width, height = self.globalRect
        start_x = x + (width * 0.2)
        stop_x = x + (width * 0.8)
        start_y = stop_y = y + (height // 2)

        self.pointing_device.drag(start_x, start_y, stop_x, stop_y)


class HistoryDelegate(ListItemWithActions):

    """Autopilot helper for the History delegate."""

    def send_message(self):
        self._show_actions()
        icon = self.select_single('Icon', name='message')
        self.pointing_device.click_object(icon)

    def _show_actions(self):
        x, y, width, height = self.globalRect
        start_x = x + (width * 0.8)
        stop_x = x + (width * 0.2)
        start_y = stop_y = y + (height // 2)

        self.pointing_device.drag(start_x, start_y, stop_x, stop_y)

    def add_contact(self):
        self._show_actions()
        icon = self.select_single('Icon', name='contact-new')
        self.pointing_device.click_object(icon)



# -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-
# Copyright 2012 Canonical
#
# This file is part of dialer-app.
#
# dialer-app is free software: you can redistribute it and/or modify it
# under the terms of the GNU General Public License version 3, as published
# by the Free Software Foundation.

"""Tests for the Dialer App"""

from autopilot.matchers import Eventually
from testtools.matchers import Equals

from dialer_app.tests import DialerAppTestCase


class TestDialer(DialerAppTestCase):
    """Tests for the Call panel."""

    # FIXME: test disabled until we get a better way to test the keypad keys
    # def test_keypad_buttons(self):
    #    keypad_entry = self.main_view.dialer_page._get_keypad_entry()
    #    keypad_keys = self.main_view.dialer_page._get_keypad_keys()
    #
    #    text = ""
    #    for key in keypad_keys:
    #        self.main_view.dialer_page.click_keypad_button(key)
    #        text += key.label
    #
    #    self.assertThat(keypad_entry.value, Eventually(Equals(text)))

    def test_erase_button(self):
        keypad_entry = self.main_view.dialer_page.dial_number("123", "1 23")
        self.main_view.dialer_page.click_erase_button()

        self.assertThat(
            keypad_entry.value,
            Eventually(Equals("12"))
        )

        self.main_view.dialer_page.click_erase_button()
        self.main_view.dialer_page.click_erase_button()

        self.assertThat(
            keypad_entry.value,
            Eventually(Equals(""))
        )

    def test_dialer_view_is_default(self):
        """Ensure the dialer view is the default view on app startup."""
        dialer_page = self.main_view.dialer_page

        self.assertThat(dialer_page.visible, Eventually(Equals(True)))

    def test_dialer_copy_and_paste(self):
        keypad_entry = self.main_view.dialer_page._get_keypad_entry()
        keypad_keys = self.main_view.dialer_page._get_keypad_keys()
        tmpKey = None
        for key in keypad_keys:
            self.main_view.dialer_page.click_keypad_button(key)
            # to be used later
            tmpKey = key

        value = keypad_entry.value
        self.main_view.dialer_page.trigger_copy_and_paste()
        self.main_view.dialer_page.trigger_select_all()
        self.main_view.dialer_page.trigger_cut()

        self.assertThat(
            keypad_entry.value,
            Eventually(Equals(""))
        )

        # trigger paste
        self.main_view.dialer_page.trigger_copy_and_paste()
        self.main_view.dialer_page.trigger_paste()

        self.assertThat(
            keypad_entry.value,
            Eventually(Equals(value))
        )

        # select all text
        self.main_view.dialer_page.trigger_copy_and_paste()

        # first tap ouside just closes the copy and paste dialog
        self.main_view.dialer_page.click_keypad_button(tmpKey)

        # now change the text
        self.main_view.dialer_page.click_keypad_button(tmpKey)

        # check if selection is gone
        self.assertThat(keypad_entry.selectedText,
            Eventually(Equals(""))
        )



# -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-
# Copyright 2015 Canonical
# Author: Tiago Salem Herrmann <tiago.herrmann@canonical.com>
#
# This file is part of dialer-app.
#
# dialer-app is free software: you can redistribute it and/or modify it
# under the terms of the GNU General Public License version 3, as published
# by the Free Software Foundation.

"""Tests for the Dialer App using ofono-phonesim"""

import os

from autopilot.matchers import Eventually
from testtools.matchers import Equals
from testtools import skipIf, skipUnless

from dialer_app.tests import DialerAppTestCase
from dialer_app import helpers
from dialer_app import fixture_setup


@skipUnless(helpers.is_phonesim_running(),
            "this test needs to run under with-ofono-phonesim")
@skipIf(os.uname()[2].endswith("maguro"),
        "tests cause Unity crashes on maguro")
class TestUSSD(DialerAppTestCase):
    """Tests for simulated ussd sessions."""

    def setUp(self):
        phonesim_modem = fixture_setup.UsePhonesimModem()
        self.useFixture(phonesim_modem)
        notification_mock = fixture_setup.MockNotificationSystem()
        self.useFixture(notification_mock)
        super().setUp()

    def tearDown(self):
        super().tearDown()

    def test_ussd_invalid_code(self):
        """Test if invalid codes are properly notified"""
        number = "*123#"
        formattedNumber = "*123#"
        self.main_view.dialer_page.dial_number(number, formattedNumber)
        self.main_view.dialer_page.click_call_button()
        self.assertThat(self.main_view.check_ussd_error_dialog_visible(),
                        Eventually(Equals(True)))

    def test_ussd_valid_code(self):
        """Test if invalid codes are properly notified"""
        number = "*225#"
        formattedNumber = "*225#"
        self.main_view.dialer_page.dial_number(number, formattedNumber)
        self.main_view.dialer_page.click_call_button()

        self.assertThat(self.main_view.check_ussd_error_dialog_visible(),
                        Equals(False))
        self.assertThat(self.main_view.check_ussd_progress_dialog_visible(),
                        Equals(False))



# -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-
# Copyright 2013 Canonical
# Author: Martin Pitt <martin.pitt@ubuntu.com>
#
# This file is part of dialer-app.
#
# dialer-app is free software: you can redistribute it and/or modify it
# under the terms of the GNU General Public License version 3, as published
# by the Free Software Foundation.

"""Tests for the Dialer App using ofono-phonesim"""

import os
import time

from autopilot.matchers import Eventually
from testtools.matchers import Equals, NotEquals, MismatchError
from testtools import skipIf, skipUnless

from dialer_app.tests import DialerAppTestCase
from dialer_app import helpers
from dialer_app import fixture_setup


@skipUnless(helpers.is_phonesim_running(),
            "this test needs to run under with-ofono-phonesim")
@skipIf(os.uname()[2].endswith("maguro"),
        "tests cause Unity crashes on maguro")
class TestCalls(DialerAppTestCase):
    """Tests for simulated phone calls."""

    def setUp(self):
        empty_history = fixture_setup.UseEmptyHistory()
        self.useFixture(empty_history)
        phonesim_modem = fixture_setup.UsePhonesimModem()
        self.useFixture(phonesim_modem)
        notification_mock = fixture_setup.MockNotificationSystem()
        self.useFixture(notification_mock)
        memory_backend = fixture_setup.UseMemoryContactBackend()
        self.useFixture(memory_backend)
        preload_data = fixture_setup.PreloadVcards()
        self.useFixture(preload_data)
        super().setUp()

    def tearDown(self):
        super().tearDown()

    @property
    def history_list(self):
        # because of the object tree, more than just one item is returned, but
        # all references point to the same item, so take the first
        return self.app.select_many(objectName="historyList")[0]

    def get_history_for_number(self, number):
        # because of the bottom edge tree structure, multiple copies of the
        # same item are returned, so just use the first one
        return self.history_list.select_many(
            "HistoryDelegate", phoneNumber=number)[0]

    def test_outgoing_noanswer(self):
        """Outgoing call to a normal number, no answer"""
        number = "144"
        formattedNumber = "1 44"
        self.main_view.dialer_page.call_number(number, formattedNumber)
        self.assertThat(
            self.main_view.live_call_page.caller, Eventually(Equals(number)))

        # If we press the hangup button too quickly it won't end the call.
        # Reported as bug http://pad.lv/1351817
        time.sleep(1)
        self.main_view.live_call_page.click_hangup_button()

        # now the live call view stays open for a few seconds after the call
        # ends, so we need to wait for the dialpad view to be active before
        # trying to interact with the bottom edge
        self.main_view.dialer_page.active.wait_for(True)

        # log should show call to the phone number
        self.main_view.dialer_page.reveal_bottom_edge_page()
        self.assertThat(self.history_list.count, Eventually(Equals(1)))
        # because of the bottom edge tree structure, multiple copies of the
        # same item are returned, so just use the first one
        self.assertThat(self.get_history_for_number(number), NotEquals(None))

    def test_outgoing_answer_local_hangup(self):
        """Outgoing call, remote answers, local hangs up"""
        # 06123xx causes accept after xx seconds
        number = "0612302"
        formattedNumber = "061-2302"

        self.main_view.dialer_page.call_number(number, formattedNumber)
        self.assertThat(
            self.main_view.live_call_page.caller, Eventually(Equals(number)))

        # stop watch should start counting
        elapsed_time = self.main_view.live_call_page.get_elapsed_call_time()
        self.assertIn("00:0", elapsed_time)

        # should still be connected after some time
        time.sleep(3)
        self.assertIn("00:0", elapsed_time)
        self.main_view.live_call_page.click_hangup_button()

    def test_outgoing_answer_remote_hangup(self):
        """Outgoing call, remote answers and hangs up"""
        number = "0512303"
        formattedNumber = "051-2303"

        # 05123xx causes immediate accept and hangup after xx seconds
        self.main_view.dialer_page.call_number(number, formattedNumber)
        self.assertThat(
            self.main_view.live_call_page.caller, Eventually(Equals(number)))

        # stop watch should start counting
        elapsed_time = self.main_view.live_call_page.get_elapsed_call_time()
        self.assertIn("00:0", elapsed_time)

        # wait for the live call view to be gone
        self.main_view.dialer_page.active.wait_for(True)

        # after remote hangs up, should switch to call log page and show call
        # to number
        self.main_view.dialer_page.reveal_bottom_edge_page()
        self.assertThat(self.history_list.visible, Eventually(Equals(True)))
        self.assertThat(self.history_list.count, Eventually(Equals(1)))
        self.assertThat(self.get_history_for_number(number), NotEquals(None))

    def test_incoming(self):
        """Incoming call"""
        number = "1234567"
        helpers.invoke_incoming_call(number)

        # wait for incoming call, accept; it would be nicer to fake-click the
        # popup notification, but as this isn't generated by dialer-app it
        # isn't exposed to autopilot
        helpers.wait_for_incoming_call()
        time.sleep(1)  # let's hear the ringing sound for a second :-)
        helpers.accept_incoming_call()

        # call back is from that number
        self.assertThat(
            self.main_view.live_call_page.caller, Eventually(Equals(number)))

        # stop watch should start counting
        elapsed_time = self.main_view.live_call_page.get_elapsed_call_time()
        self.assertIn("00:0", elapsed_time)

        try:
            self.main_view.live_call_page.click_hangup_button()
        except MismatchError as e:
            print('Expected failure due to known Mir crash '
                  '(https://launchpad.net/bugs/1240400): %s' % e)

    def test_last_called_number(self):
        number = "1234567"
        formattedNumber = "1 234-567"

        number2 = "1231234567"
        formattedNumber2 = "1 231-234-567"

        self.main_view.dialer_page.call_number(number, formattedNumber)
        time.sleep(3)
        self.main_view.live_call_page.click_hangup_button()
        self.main_view.dialer_page.active.wait_for(True)

        self.main_view.dialer_page.call_number(number2, formattedNumber2)
        time.sleep(3)
        self.main_view.live_call_page.click_hangup_button()
        self.main_view.dialer_page.active.wait_for(True)

        self.main_view.dialer_page.reveal_bottom_edge_page()
        self.assertThat(self.history_list.visible, Eventually(Equals(True)))
        self.assertThat(self.history_list.count, Eventually(Equals(1)))
        historyEntry = self.get_history_for_number(number)
        self.main_view._click_button(historyEntry)
        self.assertThat(
            self.main_view.dialer_page.dialNumber,
            Eventually(Equals(formattedNumber2)))

    def test_call_ending_goes_to_dialpad(self):
        """Make sure that when a call ends, the app goes back to dialpad"""
        number = "1234567"
        helpers.invoke_incoming_call(number)

        # wait for incoming call, accept; it would be nicer to fake-click the
        # popup notification, but as this isn't generated by dialer-app it
        # isn't exposed to autopilot
        helpers.wait_for_incoming_call()
        time.sleep(1)  # let's hear the ringing sound for a second :-)
        helpers.accept_incoming_call()

        # call back is from that number
        self.assertThat(
            self.main_view.live_call_page.caller, Eventually(Equals(number)))

        # now enter the contacts page and select the first contact
        self.main_view.live_call_page.click_new_call_button()
        self.main_view.contacts_page.open_contact(0)

        # at this point we have a pagestack full of pages, so hanging up the
        # call needs to clear that up
        helpers.hangup_call()

        # check that the dialpad view is there
        self.assertThat(self.main_view.dialer_page.active,
                        Eventually(Equals(True)))
